/*
 * Copyright 2007 Etienne Studer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.studer.idea.notifier;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.intellij.openapi.util.BuildNumber;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import com.intellij.ide.plugins.IdeaPluginDescriptor;
import com.intellij.ide.plugins.PluginManager;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationInfo;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.extensions.PluginId;

import static nu.studer.idea.notifier.PluginUpdateNotifierConfigurable.DAILY;
import static nu.studer.idea.notifier.PluginUpdateNotifierConfigurable.WEEKLY;

/**
 * This class launches a background thread which queries the IDEA plugin repository in certain intervals, as defined by the
 * <code>PluginUpdateNotifierConfigurable</code>. If some of the locally installed plugins are outofdate or an expection occurs during the netweork
 * activities, all registered listeners are notified.
 *
 * @author <a href="mailto:intellij@studer.nu">Etienne Studer</a>, Nov 7, 2006
 * @see PluginUpdateNotifierConfigurable
 */
public class PluginUpdateNotifierBackgroundComponent implements ApplicationComponent {
    private static final Logger LOG = Logger.getInstance(PluginUpdateNotifierBackgroundComponent.class.getName());

    private PluginRepositoryManagerRunnable _runnable;
    private UpdateState _updateState;
    private List<IPluginUpdateListener> _listeners;

    public static PluginUpdateNotifierBackgroundComponent getInstance() {
        return ApplicationManager.getApplication().getComponent(PluginUpdateNotifierBackgroundComponent.class);
    }

    @NotNull
    public String getComponentName() {
        return "PluginUpdateNotifier.PluginUpdateNotifierBackgroundComponent";
    }

    public void initComponent() {
        _runnable = new PluginRepositoryManagerRunnable();
        _updateState = new UpdateState(null, null);
        _listeners = new ArrayList<IPluginUpdateListener>();

        // start the background thread which queries the plugin repository manager
        Thread thread = new Thread(_runnable);
        thread.setPriority(Thread.NORM_PRIORITY - 1);
        thread.start();
    }

    public void disposeComponent() {
        _runnable.stop();
        _runnable = null;

        _listeners.clear();
    }

    @SuppressWarnings({"AnalyzingLoggingWithoutLogLevelCheck"})
    public void queryPluginRepositoryManager() {
        LOG.debug("About to iterate over installed plugins...");

        // initialize the plugin repository manager
        PluginRepositoryManager repositoryManager;
        try {
            BuildNumber ideaBuildVersion = ApplicationInfo.getInstance().getBuild();
            repositoryManager = new PluginRepositoryManager(ideaBuildVersion);
        } catch (Exception e) {
            LOG.info("Could not initialize plugin repository manager", e);
            updateException(e);// do not reset the list of outdated plugins
            return;
        }

        // give other threads a chance to run
        Thread.yield();

        final List<PluginRepositoryManager.PluginRepositoryQueryResult> outOfDatePlugins =
                new ArrayList<PluginRepositoryManager.PluginRepositoryQueryResult>();

        // get all disabled plugins
        List<String> disabledPlugins = PluginManager.getDisabledPlugins();
        LOG.debug("Disabled plugins: " + disabledPlugins);

        // configurable by the user whether to exclude disabled plugins
        PluginUpdateNotifierConfigurable notifierConfigurable = PluginUpdateNotifierConfigurable.getInstance();
        boolean ignoreDisabledPlugins = notifierConfigurable.IGNORE_DISABLED_PLUGINS;
        boolean ignoreBundledPlugins = notifierConfigurable.IGNORE_BUNDLED_PLUGINS;
        LOG.debug(ignoreDisabledPlugins ? "Ignore disabled plugins" : "Do not ignore disabled plugins");

        // iterate over all locally installed plugins and check for new repository version
        Application application = ApplicationManager.getApplication();
        IdeaPluginDescriptor[] descriptors = application.getPlugins();
        for (IdeaPluginDescriptor descriptor : descriptors) {
            PluginId pluginId = descriptor.getPluginId();
            LOG.debug("Found plugin with id " + pluginId);

            if (ignoreDisabledPlugins) {
                if (disabledPlugins.contains(pluginId.getIdString())) {
                    LOG.debug("Ignore disabled plugin with id " + pluginId);
                    continue;
                }
            }

            //boaz:Ignore bundled
            if (ignoreBundledPlugins) {
                if (descriptor.isBundled()) {
                    LOG.debug("Ignore bundled plugin with id " + pluginId);
                    continue;
                }

            }


            PluginRepositoryManager.PluginRepositoryQueryResult result = repositoryManager.isNewVersionAvailable(descriptor);
            if (result.isNewVersionAvailable()) {
                LOG.info("New plugin version available for plugin with id " + pluginId);
                outOfDatePlugins.add(result);
            }
        }

        setUpdateState(new UpdateState(Collections.unmodifiableList(outOfDatePlugins), null));

        LOG.debug("...finished iterating over installed plugins");
    }

    public void applyConfigurationChange() {
        synchronized (_runnable.LOCK) {
            _runnable.LOCK.notifyAll();
        }
    }

    public UpdateState getUpdateState() {
        synchronized (this) {
            return _updateState;
        }
    }

    private void setUpdateState(UpdateState inUpdateState) {
        UpdateState newUpdateState;
        synchronized (this) {
            _updateState = inUpdateState;
            newUpdateState = _updateState;
        }
        notifyListeners(newUpdateState);
    }

    private void updateException(Exception inException) {
        UpdateState newUpdateState;
        synchronized (this) {
            _updateState = new UpdateState(_updateState.getOutOfDatePlugins(), inException);
            newUpdateState = _updateState;
        }
        notifyListeners(newUpdateState);
    }

    private void notifyListeners(UpdateState inUpdateState) {
        IPluginUpdateListener[] listeners = _listeners.toArray(new IPluginUpdateListener[_listeners.size()]);
        for (IPluginUpdateListener listener : listeners) {
            listener.pluginsUpdated(inUpdateState);
        }
    }

    public void addUpdateListener(IPluginUpdateListener inListener) {
        _listeners.add(inListener);
    }

    public void removeUpdateListener(IPluginUpdateListener inListener) {
        _listeners.remove(inListener);
    }

    public interface IPluginUpdateListener {
        void pluginsUpdated(UpdateState inUpdateState);
    }

    public static class UpdateState {
        @Nullable
        private List<PluginRepositoryManager.PluginRepositoryQueryResult> _outOfDatePlugins;
        @Nullable
        private Exception _exception;

        private UpdateState(@Nullable List<PluginRepositoryManager.PluginRepositoryQueryResult> inOutOfDatePlugins, @Nullable Exception inException) {
            _outOfDatePlugins = inOutOfDatePlugins;
            _exception = inException;
        }

        @Nullable
        public List<PluginRepositoryManager.PluginRepositoryQueryResult> getOutOfDatePlugins() {
            return _outOfDatePlugins;
        }

        @Nullable
        public Exception getException() {
            return _exception;
        }
    }

    private class PluginRepositoryManagerRunnable implements Runnable {
        @SuppressWarnings({"AnalyzingVariableNaming"})
        private final Object LOCK = new Object();

        private boolean _doRun = true;

        public void run() {
            LOG.info("PluginRepositoryManagerRunnable started...");
            while (_doRun) {
                PluginUpdateNotifierConfigurable notifierConfigurable = PluginUpdateNotifierConfigurable.getInstance();
                if (notifierConfigurable.CHECK_UPDATES) {
                    if (isReadyForNextExecution()) {
                        LOG.debug("About to query PluginRepositoryManager...");
                        queryPluginRepositoryManager();

                        // update timestamps, taking into account whether plugin manager activities have been successful or not
                        UpdateState updateState = getUpdateState();
                        updateNextExecutionTime(updateState.getException() == null);

                        LOG.debug("...finished querying PluginRepositoryManager");
                    }
                    waitForNextExecution();
                } else {
                    waitIndeterminately();
                }
            }
            LOG.info("...PluginRepositoryManagerRunnable stopped");
        }

        private boolean isReadyForNextExecution() {
            PluginUpdateNotifierConfigurable notifierConfigurable = PluginUpdateNotifierConfigurable.getInstance();
            if (notifierConfigurable.NEXT_EXECUTION <= System.currentTimeMillis()) {
                return true;
            }

            // for daily and weekly update intervals, a match on the same day is also okay no matter what the current time is
            if (DAILY.equals(notifierConfigurable.UPDATE_INTERVAL) || WEEKLY.equals(notifierConfigurable.UPDATE_INTERVAL)) {
                if (PluginUpdateNotifierUtil.onSameDayOrBefore(notifierConfigurable.NEXT_EXECUTION, System.currentTimeMillis())) {
                    return true;
                }
            }

            return false;
        }

        private void updateNextExecutionTime(boolean inSuccessful) {
            PluginUpdateNotifierConfigurable notifierConfigurable = PluginUpdateNotifierConfigurable.getInstance();
            notifierConfigurable.updateNextExecutionTime(inSuccessful);
        }

        @SuppressWarnings({"AnalyzingLoggingWithoutLogLevelCheck"})
        private void waitForNextExecution() {
            try {
                PluginUpdateNotifierConfigurable notifierConfigurable = PluginUpdateNotifierConfigurable.getInstance();
                final long waitTime = Math.max(1, notifierConfigurable.NEXT_EXECUTION - System.currentTimeMillis());

                LOG.debug("About to wait for " + waitTime + " ms...");
                synchronized (LOCK) {
                    LOCK.wait(waitTime);
                }
                LOG.debug("...finished waiting");
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        }

        private void waitIndeterminately() {
            try {
                LOG.debug("About to wait indeterminately...");
                synchronized (LOCK) {
                    LOCK.wait();
                }
                LOG.debug("...finished waiting indeterminately");
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        public void stop() {
            LOG.debug("Signal to stop background thread...");
            _doRun = false;
            synchronized (LOCK) {
                LOCK.notifyAll();
            }
        }
    }
}
