/*
 * Copyright 2007 Etienne Studer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.studer.idea.notifier;

import java.text.SimpleDateFormat;
import javax.swing.*;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
// boaz was:
//import com.intellij.util.ObjectUtils;
//boaz now:
import org.apache.commons.lang.ObjectUtils;


/**
 * This class defines the UI for the plugin configuration and is backed by an IDEA Form.
 *
 * @author <a href="mailto:estuder@navis.com">Etienne Studer</a>, Nov 15, 2006
 */
public class PluginUpdateNotifierConfigurationPane {
    @NonNls
    private static final SimpleDateFormat DATE_ONLY = new SimpleDateFormat("EEEE, MMMM d, yyyy");
    @NonNls
    private static final SimpleDateFormat DATE_TIME = new SimpleDateFormat("EEEE, MMMM d HH:mm, yyyy");

    protected JPanel _rootComponent;
    protected JTextArea _descriptionTextArea;
    protected JCheckBox _checkUpdatesCheckBox;
    protected JComboBox _intervalComboBox;
    protected JCheckBox _ignoreDisabledPluginsCheckBox;
    private JCheckBox _ignoreBundledPluginsCheckBox;
    protected JButton _checkNowButton;
    protected JLabel _lastUpdateCheckLabel;
    protected JLabel _lastUpdateCheckDateLabel;
    protected JLabel _nextUpdateCheckLabel;
    protected JLabel _nextUpdateCheckDateLabel;

    private boolean _showTime;

    public PluginUpdateNotifierConfigurationPane() {
        //noinspection HardCodedStringLiteral
        this(new String[]{"alpha", "beta", "gamma", "delta"});
    }

    public PluginUpdateNotifierConfigurationPane(@NotNull String[] inIntervals) {
        _intervalComboBox.setModel(new DefaultComboBoxModel(inIntervals));
        _descriptionTextArea.setFont(_lastUpdateCheckLabel.getFont());  // assure same font on text area
        _showTime = false;
    }

    public JPanel getRootComponent() {
        return _rootComponent;
    }

    public JButton getCheckNowButton() {
        return _checkNowButton;
    }

    public boolean isShowTime() {
        return _showTime;
    }

    public void setShowTime(boolean inShowTime) {
        _showTime = inShowTime;
    }

    public void getData(PluginUpdateNotifierConfigurable inNotifierConfigurable) {
        inNotifierConfigurable.CHECK_UPDATES = isUpdateCheckEnabled();
        inNotifierConfigurable.UPDATE_INTERVAL = getSelectedInterval();
        inNotifierConfigurable.IGNORE_DISABLED_PLUGINS = isIgnoreDisabledPluginsEnabled();
        inNotifierConfigurable.IGNORE_BUNDLED_PLUGINS = isIgnoreBundledPluginsEnabled();
    }

    public void setData(PluginUpdateNotifierConfigurable inNotifierConfigurable) {
        setUpdateCheckEnabled(inNotifierConfigurable.CHECK_UPDATES);
        setSelectedInterval(inNotifierConfigurable.UPDATE_INTERVAL);
        setIgnoreDisabledPluginsEnabled(inNotifierConfigurable.IGNORE_DISABLED_PLUGINS);
        setIgnoreBundledPluginsEnabled(inNotifierConfigurable.IGNORE_BUNDLED_PLUGINS);
        setLastUpdateCheckDate(inNotifierConfigurable.LAST_EXECUTION);
        setNextUpdateCheckDate(inNotifierConfigurable.NEXT_EXECUTION);
    }

    public boolean isModified(PluginUpdateNotifierConfigurable inNotifierConfigurable) {
        return !ObjectUtils.equals(inNotifierConfigurable.CHECK_UPDATES, isUpdateCheckEnabled()) ||
                PluginUpdateNotifierUtil.doDiffer(inNotifierConfigurable.UPDATE_INTERVAL, getSelectedInterval()) ||
                !ObjectUtils.equals(inNotifierConfigurable.IGNORE_DISABLED_PLUGINS, isIgnoreDisabledPluginsEnabled()) ||
               !ObjectUtils.equals(inNotifierConfigurable.IGNORE_BUNDLED_PLUGINS, isIgnoreBundledPluginsEnabled());

    }

    private boolean isUpdateCheckEnabled() {
        return _checkUpdatesCheckBox.isSelected();
    }

    public void setUpdateCheckEnabled(boolean inEnable) {
        _checkUpdatesCheckBox.setSelected(inEnable);
    }

    private String getSelectedInterval() {
        return (String) _intervalComboBox.getSelectedItem();
    }

    private void setSelectedInterval(String inInterval) {
        _intervalComboBox.setSelectedItem(inInterval);
    }

    public void setIgnoreDisabledPluginsEnabled(boolean inEnable) {
        _ignoreDisabledPluginsCheckBox.setSelected(inEnable);
    }

    public void setIgnoreBundledPluginsEnabled(boolean inEnable) {
        _ignoreBundledPluginsCheckBox.setSelected(inEnable);
    }

    public boolean isIgnoreDisabledPluginsEnabled() {
        return _ignoreDisabledPluginsCheckBox.isSelected();
    }

    public boolean isIgnoreBundledPluginsEnabled() {
        return _ignoreBundledPluginsCheckBox.isSelected();
    }

    private void setLastUpdateCheckDate(long inDate) {
        String text = null;
        if (inDate != 0) {
            text = _showTime ? DATE_TIME.format(inDate) : DATE_ONLY.format(inDate);
        }
        _lastUpdateCheckDateLabel.setText(text);
    }

    private void setNextUpdateCheckDate(long inDate) {
        String text = null;
        if (inDate != 0) {
            text = _showTime ? DATE_TIME.format(inDate) : DATE_ONLY.format(inDate);
        }
        _nextUpdateCheckDateLabel.setText(text);
    }

    public static void main(String[] inArgs) {
        @NonNls JFrame frame = new JFrame("PluginUpdateNotifierConfigurationPane");
        frame.setContentPane(new PluginUpdateNotifierConfigurationPane()._rootComponent);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }
}
