/*
 * Copyright 2007 Etienne Studer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.studer.idea.notifier;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;
import javax.swing.*;

import org.jetbrains.annotations.NotNull;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.ui.LightColors;

/**
 * This class defines the custom indication component displayed in the status bar. When the <code>PluginUpdateNotifierBackgroundComponent</code> fires
 * an event about outofdate plugins, this component updates its icon appropriately and shows the IDEA notification popup.
 *
 * @author <a href="mailto:intellij@studer.nu">Etienne Studer</a>, Nov 7, 2006
 */
public class PluginUpdateNotifierIndicationComponent implements ProjectComponent {
    private static final Logger LOG = Logger.getInstance(PluginUpdateNotifierIndicationComponent.class.getName());

    private static final Icon UPTODATE_ICON = IconLoader.getIcon("/nodes/plugin.png");
    private static final Icon OUTOFDATE_ICON = IconLoader.getIcon("/nodes/pluginobsolete.png");

    private Project _project;
    private JLabel _notificationLabel;
    private PluginUpdateNotifierBackgroundComponent.IPluginUpdateListener _listener;
    private boolean _customComponentAdded;

    public PluginUpdateNotifierIndicationComponent(Project inProject) {
        _project = inProject;
    }

    @NotNull
    public String getComponentName() {
        return "PluginUpdateNotifier.PluginUpdateNotifierIndicationComponent";
    }

    public void initComponent() {
    }

    public void disposeComponent() {
    }

    public void projectOpened() {
        _notificationLabel = new JLabel();
        _notificationLabel.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent inEvent) {
                PluginUpdateNotifierDialog dialog = new PluginUpdateNotifierDialog(_project);
                dialog.show();
            }
        });

        _listener = new PluginUpdateNotifierBackgroundComponent.IPluginUpdateListener() {
            public void pluginsUpdated(PluginUpdateNotifierBackgroundComponent.UpdateState inUpdateState) {
                refreshIndicationComponent(inUpdateState);
            }
        };

        _customComponentAdded = false;

        // add a listener that is notified about plugin repository changes
        getBackgroundComponent().addUpdateListener(_listener);

        // notification will take place in certain intervals only, but we want the view to be initialized properly in the beginning already
        // most important, this will make sure that all projects are in-synch when consecutive projects are opened
        PluginUpdateNotifierBackgroundComponent.UpdateState updateState = getBackgroundComponent().getUpdateState();
        _listener.pluginsUpdated(updateState);
    }

    public void projectClosed() {
        getBackgroundComponent().removeUpdateListener(_listener);
    }

    private void installCustomIndicationComponentIfNeeded() {
        // install the label with the plugin status icon in the status bar
        if (!_customComponentAdded) {
            StatusBar statusBar = WindowManager.getInstance().getStatusBar(_project);
            statusBar.addCustomIndicationComponent(_notificationLabel);
            _customComponentAdded = true;
        }
    }

    private void refreshIndicationComponent(PluginUpdateNotifierBackgroundComponent.UpdateState inUpdateState) {
        LOG.debug("About to iterate over installed plugins...");

        final List<PluginRepositoryManager.PluginRepositoryQueryResult> outOfDatePlugins = inUpdateState.getOutOfDatePlugins();
        final Exception exception = inUpdateState.getException();

        runInDispatchThread(new Runnable() {
            public void run() {
                // 1) set icon according to the plugins' state
                if (outOfDatePlugins == null) {
                    // show no icon if the list of outdated plugins is null, since this means the state of the plugins has not been queried, yet
                    _notificationLabel.setIcon(null);
                } else if (outOfDatePlugins.size() == 0) {
                    // set icon such that it reflects uptodate state
                    _notificationLabel.setIcon(UPTODATE_ICON);
                } else {
                    // set icon such that it reflects outofdate state
                    _notificationLabel.setIcon(OUTOFDATE_ICON);
                }

                // 2) disable the icon in case an exception happened recently while the plugin repository was accessed
                if (exception != null) {
                    Icon disabledIcon = IconLoader.getDisabledIcon(_notificationLabel.getIcon());
                    _notificationLabel.setIcon(disabledIcon);
                }

                // 3) if the icon is not null, make sure the label with the plugin status icon is installed in the status bar
                if (_notificationLabel.getIcon() != null) {
                    installCustomIndicationComponentIfNeeded();
                }

                // 4) if some plugins are outofdate, fire a message in the notification popup
                if (outOfDatePlugins != null && outOfDatePlugins.size() > 0) {
                    JTextArea area = new JTextArea();
                    area.setOpaque(false);
                    area.setEditable(false);

                    StringBuffer notification = PluginUpdateNotifierUtil.createNotificationMessage(outOfDatePlugins, false);
                    area.setText(notification.toString());

                    StatusBar statusBar = WindowManager.getInstance().getStatusBar(_project);
                    if (statusBar != null) {// can be null if project has just been closed but task is already scheduled
                        statusBar.fireNotificationPopup(area, LightColors.BLUE);
                        if (LOG.isDebugEnabled()) {
                            LOG.debug("Firing notification popup >" + area.getText() + "<");
                        }
                    }
                }
            }
        });

        LOG.debug("...finished iterating over installed plugins");
    }

    private static void runInDispatchThread(Runnable inRunnable) {
        ApplicationManager.getApplication().invokeLater(inRunnable);
    }

    private static PluginUpdateNotifierBackgroundComponent getBackgroundComponent() {
        Application application = ApplicationManager.getApplication();
        PluginUpdateNotifierBackgroundComponent backgroundComponent = application.getComponent(PluginUpdateNotifierBackgroundComponent.class);
        return backgroundComponent;
    }
}
