/*
 * Copyright 2007 Etienne Studer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.studer.idea.notifier;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
// boaz was:
//import com.intellij.util.ObjectUtils;
//boaz now:
import org.apache.commons.lang.ObjectUtils;

/**
 * This class contains some utility methods.
 *
 * @author <a href="mailto:estuder@navis.com">Etienne Studer</a>, Nov 14, 2006
 */
public class PluginUpdateNotifierUtil {
    @NotNull
    public static StringBuffer createNotificationMessage(@NotNull List<PluginRepositoryManager.PluginRepositoryQueryResult> inOutOfDatePlugins,
                                                         boolean inVerbose) {
        StringBuffer notification = new StringBuffer();

        // create shallow copy to make sure we don't modify the pased-in list
        List<PluginRepositoryManager.PluginRepositoryQueryResult> outOfDatePlugins =
                new ArrayList<PluginRepositoryManager.PluginRepositoryQueryResult>(inOutOfDatePlugins);

        if (outOfDatePlugins.size() > 0) {
            // static text
            if (outOfDatePlugins.size() == 1) {
                String msg = inVerbose ? PluginUpdateNotifierBundle.message("new.plugin.version.available.verbose") :
                        PluginUpdateNotifierBundle.message("new.plugin.version.available");
                notification.append(msg).append("\n\n");
            } else {
                String msg = inVerbose ? PluginUpdateNotifierBundle.message("new.plugin.versions.available.verbose") :
                        PluginUpdateNotifierBundle.message("new.plugin.versions.available");
                notification.append(msg).append("\n\n");
            }

            // sort by plugin name
            Collections.sort(outOfDatePlugins, new PluginRepositoryQueryResultComparator());

            // name and version of outofdate plugins
            for (int i = 0; i < outOfDatePlugins.size(); i++) {
                PluginRepositoryManager.PluginRepositoryQueryResult result = outOfDatePlugins.get(i);
                String name = result.getCurrentDescriptor().getName();
                String version = result.getNewVersion();
                notification.append(i > 0 ? "\n" : "").append(" - ").append(name).append(" ").append(version);
            }
        } else {
            String msg = inVerbose ? PluginUpdateNotifierBundle.message("no.new.plugin.versions.available.verbose") :
                    PluginUpdateNotifierBundle.message("no.new.plugin.versions.available");
            notification.append(msg);
        }

        return notification;
    }

    /**
     * Determines whether two Strings are different, treating empty strings and null strings as being equal.
     *
     * @param inFirst  the first of the two strings to compare
     * @param inSecond the second of the two strings to compare
     * @return <i>true</i> if the two specified strings differ (handling null and empty strings as equal)
     */
    public static boolean doDiffer(@Nullable String inFirst, @Nullable String inSecond) {
        boolean equalNullStrs = ("".equals(inFirst) && null == inSecond) || ("".equals(inSecond) && null == inFirst);
        return !equalNullStrs && !ObjectUtils.equals(inFirst, inSecond);
    }

    /**
     * Compares a timestamp with a reference timestamp to be on the same calendar day or before regardless of their time of day. The default time zone
     * is applied.
     *
     * @param inTimestamp          the timestamp to compare if on the same day or before the reference timestamp
     * @param inReferenceTimestamp the reference timestamp
     * @return <i>true</i> if the first timestamp is on the same calendar day or before the reference timestamp in the default time zone
     */
    public static boolean onSameDayOrBefore(long inTimestamp, long inReferenceTimestamp) {
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(inTimestamp);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);

        Calendar calRef = Calendar.getInstance();
        calRef.setTimeInMillis(inReferenceTimestamp);
        calRef.set(Calendar.HOUR_OF_DAY, 0);
        calRef.set(Calendar.MINUTE, 0);
        calRef.set(Calendar.SECOND, 0);
        calRef.set(Calendar.MILLISECOND, 0);

        boolean sameDayOrBefore = cal.compareTo(calRef) <= 0;
        return sameDayOrBefore;
    }

    public static final long ONE_HOUR = 1000 * 60 * 60;
    public static final long TWO_HOURS = ONE_HOUR * 2;
    public static final long ONE_DAY = ONE_HOUR * 24;
    public static final long ONE_WEEK = ONE_DAY * 7;
}
